﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using VsliveOrd.Api;

namespace VsliveOrd.UnitTests
{
    [TestClass]
    public class PersonServiceFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            _RepositoryInstance = null;
            _ValidatorInstance = null;
        }


        private FakePersonValidator _ValidatorInstance;
        private FakePersonRepository _RepositoryInstance;
        private PersonService _SystemUnderTest;
        public PersonService SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest =
                        new PersonService(RepositoryInstance,
                        ValidatorInstance
                        );
                }

                return _SystemUnderTest;
            }
        }


        public FakePersonValidator ValidatorInstance
        {
            get
            {
                if (_ValidatorInstance == null)
                {
                    _ValidatorInstance = new FakePersonValidator();
                }

                return _ValidatorInstance;
            }
        }



        public FakePersonRepository RepositoryInstance
        {
            get
            {
                if (_RepositoryInstance == null)
                {
                    _RepositoryInstance = new FakePersonRepository();
                }

                return _RepositoryInstance;
            }
        }


        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SaveDoesNotGetCalledForInvalidPerson()
        {
            // Person temp = new Person();
            ValidatorInstance.ReturnValue = false;

            SystemUnderTest.Save(new Person());

            Assert.IsFalse(RepositoryInstance.WasSaveCalled,
                "Person was invalid.  Save shouldn't be called.");
        }


        [TestMethod]
        public void SaveIsCalledForValidPerson()
        {
            //    Person temp = new Person();

            //    temp.FirstName = "ben";
            //    temp.LastName = "day";

            ValidatorInstance.ReturnValue = true;

            SystemUnderTest.Save(new Person());

            Assert.IsTrue(RepositoryInstance.WasSaveCalled,
                "Person was valid.  Save should be called.");
        }


    }
}
